﻿using System;
using System.Linq;
using System.Windows.Forms;

using System.IO;
using MonoTorrent.Common;

namespace NBTorrentCreator {
    public partial class NBMainForm : Form {
        private bool creationAborted = false;
        private TorrentCreator torrCreator = null;
        private IAsyncResult creationResult = null;

        public NBMainForm() {
            InitializeComponent();
            this.Text = "NB TorrentCreator v" + Application.ProductVersion;
            cbPieceSize.SelectedIndex = 0;
        }

        private void bCreate_Click(object sender, EventArgs e) {
            //Error Check the required values
            if(String.IsNullOrWhiteSpace(tbSource.Text) || !Directory.Exists(tbSource.Text) && !File.Exists(tbSource.Text)) {
                MessageBox.Show(this, "Source not valid.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            //Create our torrent
            torrCreator = new TorrentCreator();

            if(!String.IsNullOrWhiteSpace(tbAnnounceList.Text))
                foreach(string A in tbAnnounceList.Lines)
                    if(!String.IsNullOrWhiteSpace(A))
                        torrCreator.Announces.Add(A.Split(new char[] { '|' }, StringSplitOptions.RemoveEmptyEntries).ToList());
                        //torrCreator.Announces.Add(new MonoTorrent.RawTrackerTier(A.Split(new char[] {'|'}, StringSplitOptions.RemoveEmptyEntries)));
            if(!String.IsNullOrWhiteSpace(tbComment.Text))
                torrCreator.Comment = tbComment.Text;
            if(!String.IsNullOrWhiteSpace(tbCreatedBy.Text))
                torrCreator.CreatedBy = tbCreatedBy.Text;
            if(!String.IsNullOrWhiteSpace(tbPublisher.Text))
                torrCreator.Publisher = tbPublisher.Text;
            if(!String.IsNullOrWhiteSpace(tbPublisherUrl.Text))
                torrCreator.PublisherUrl = tbPublisherUrl.Text;
            if(cbPieceSize.SelectedIndex != 0)
                torrCreator.PieceLength = (long)Math.Pow(2, cbPieceSize.SelectedIndex + 13);
            if(!String.IsNullOrWhiteSpace(tbCustom.Text))
                foreach(string A in tbCustom.Lines)
                    if(!String.IsNullOrWhiteSpace(A)) {
                        string[] customValue = A.Split(new char[] { '|' }, StringSplitOptions.RemoveEmptyEntries);
                        if(customValue.Length != 2) {
                            MessageBox.Show(this, "Custom value not valid.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                            return;
                        }                        
                        //torrCreator.SetCustom(customValue[0], new MonoTorrent.BEncoding.BEncodedString(customValue[1]));
                    }
            if(!String.IsNullOrWhiteSpace(tbCustomSecure.Text))
                foreach(string A in tbCustomSecure.Lines)
                    if(!String.IsNullOrWhiteSpace(A)) {
                        string[] customValue = A.Split(new char[] { '|' }, StringSplitOptions.RemoveEmptyEntries);
                        if(customValue.Length != 2) {
                            MessageBox.Show(this, "Custom value not valid.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                            return;
                        }
                        //torrCreator.SetCustomSecure(customValue[0], new MonoTorrent.BEncoding.BEncodedString(customValue[1]));
                    }


            torrCreator.Private = cbPrivate.Checked;
            torrCreator.StoreMD5 = cbStoreMD5.Checked;

            //Create the update progress function
            torrCreator.Hashed += delegate(object o, TorrentCreatorEventArgs e2) {
                if(!Double.IsNaN(e2.OverallCompletion))
                    pbOverall.Invoke((Action)(() => pbOverall.Value = e2.OverallCompletion >= pbOverall.Maximum ? pbOverall.Maximum : (e2.OverallCompletion <= pbOverall.Minimum ? pbOverall.Minimum : (int)e2.OverallCompletion)));
            };


            //Reset the progress state for a new run, and start saving the torrent asynchronously
            bClose.Text = "Cancel";
            gbSource.Enabled = gbProperties.Enabled = gbOther.Enabled = false;
            bCreate.Enabled = false;
            creationResult = torrCreator.BeginCreate(new NBTorrentFileSource(tbSource.Text, cbIgnoreHidden.Checked, tbSkipFiles.Text), TorrentCreated, torrCreator);
        }

        private void TorrentCreated(IAsyncResult result) {
            //Invoke the main thread and reset the gui\save the torrent
            this.Invoke((Action)(() => {
                //Save the result if needed
                try {
                    //Save the torrent file
                    if(result.IsCompleted && !creationAborted)
                        using(SaveFileDialog saveFile = new SaveFileDialog() { FileName = Path.GetFileName(tbSource.Text) + ".torrent", InitialDirectory = Path.GetDirectoryName(tbSource.Text + ".fileparentdirectory"), CheckFileExists = false, OverwritePrompt = true, DefaultExt = "torrent", AddExtension = true, Filter = "torrent files (*.torrent)|*.torrent|All files (*.*)|*.*", FilterIndex = 0 })
                            if(saveFile.ShowDialog(this) == DialogResult.OK)
                                (result.AsyncState as TorrentCreator).EndCreate(result, saveFile.FileName);
                } catch(Exception ex) {
                    MessageBox.Show(this, "Error creating torrent: " + ex + ".", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }

                //Have the main forum reset the gui
                pbOverall.Value = pbOverall.Minimum;
                creationAborted = false;

                bClose.Text = "Close";
                gbSource.Enabled = gbProperties.Enabled = gbOther.Enabled = true;
                bCreate.Enabled = true;
                creationResult = null;
            }));
        }

        private void bFile_Click(object sender, EventArgs e) {
            using(OpenFileDialog sourceFile = new OpenFileDialog() { CheckFileExists = true, Multiselect = false })
                if(sourceFile.ShowDialog(this) == DialogResult.OK)
                    tbSource.Text = sourceFile.FileName;
        }

        private void bFolder_Click(object sender, EventArgs e) {
            using(FolderBrowserDialog sourceFolder = new FolderBrowserDialog() { ShowNewFolderButton = false, RootFolder = Environment.SpecialFolder.MyComputer, Description = "Choose a source folder." })
                if(sourceFolder.ShowDialog(this) == DialogResult.OK)
                    tbSource.Text = sourceFolder.SelectedPath;
        }

        private void bClose_Click(object sender, EventArgs e) {
            if(creationResult != null) {
                if(!creationResult.IsCompleted && !creationAborted) {
                    creationAborted = true;
                    torrCreator.AbortCreation();
                }
            } else {
                Application.Exit();
            }
        }

        private void NBMainForm_FormClosing(object sender, FormClosingEventArgs e) {
            if(creationResult != null)
                bClose_Click(sender, null);
        }
    }
}
